;****************************************************************************
;Surface Normal
;A(x0,y0,z0), B(X1,Y1,Z1), C(X2,Y2,Z2)
;the vector normal = AB^AC = (Xn,Yn,Zn) with
;        Xn=(y2-y1)(z3-z1)-(z2-z1)(y3-y1)
;        Yn=(z2-z1)(x3-x1)-(x2-x1)(z3-z1)
;        Zn=(x2-x1)(y3-y1)-(y2-y1)(x3-x1)
;negative norm means facing you, positive means don't draw it!
;probably want to pre-calculate, and then rotate?

FindZNormal proc	;40 clock cycles
	;accepts params in pt1x, pt1y, pt1z, pt2x, ...
	;returns in ecx

	mov ax,[pt2x]		;load
	mov bx,[pt3y]		;
	sub ax,[pt1x]		;x2 - x1
	sub bx,[pt1y]		;y3 - y1
	imul bx			;(x1-x2)(y3-y1) -> dx:ax
	mov cx,dx		;move dx:ax -> ecx
	mov bx,[pt2y]		;load
	shl ecx,16		;move dx:ax -> ecx
	mov cx,ax		;move dx:ax -> ecx
	mov ax,[pt3x]		;load
	sub bx,[pt1y]		;y2 - y1
	sub ax,[pt1x]		;x1 - x3
	imul bx			;(y2-y1)(x1-x3)
	shl edx,16		;prepare for ecx - edx
	mov dx,ax		;""
	sub ecx,edx		;(x1-x2)(y1-y3)-(y2-y1)(x1-x3)
   neg ecx ;reversed z
	ret
ENDP


FindYNormal proc
	;accepts params in pt1x, pt1y, pt1z, pt2x, ...
	;returns in ecx

	mov ax,[pt2z]		;load
	mov bx,[pt3x]		;
	sub ax,[pt1z]		;z2 - z1
	sub bx,[pt1x]		;x1 - x3
	imul bx			;(x2-x1)(y1-y3) -> dx:ax
	mov cx,dx		;move dx:ax -> ecx
	mov bx,[pt2x]		;load
	shl ecx,16		;move dx:ax -> ecx
	mov cx,ax		;move dx:ax -> ecx
	mov ax,[pt3z]		;load
	sub bx,[pt1x]		;x2 - x1
	sub ax,[pt1z]		;z2 - z3
	imul bx			;(y2-y1)(x1-x3)
	shl edx,16		;prepare for ecx - edx
	mov dx,ax		;""
	sub ecx,edx		;(z2-z1)(x1-x3)-(x2-x1)(z1-z3)
	ret
ENDP

FindXNormal proc
	;accepts params in pt1x, pt1y, pt1z, pt2x, ...
	;returns in ecx

	mov ax,[pt2y]		;load
	mov bx,[pt3z]		;
	sub ax,[pt1y]		;y1 - y2
	sub bx,[pt1z]		;z1 - z3
	imul bx			;(y1-y2)(z1-z3) -> dx:ax
	mov cx,dx		;move dx:ax -> ecx
	mov bx,[pt2z]		;load
	shl ecx,16		;move dx:ax -> ecx
	mov cx,ax		;move dx:ax -> ecx
	mov ax,[pt3y]		;load
	sub bx,[pt1z]		;z1 - z2
	sub ax,[pt1y]		;y1 - y3
	imul bx			;(z1-z2)(y1-y3)
	shl edx,16		;prepare for ecx - edx
	mov dx,ax		;""
	sub ecx,edx		;(y1-y2)(z1-z3)-(z1-z2)(y1-y3)
	ret
ENDP


;****************************************************************************
;Trig
;	sets up the rotation variables (use for each different angle)
;	this speeds up the object drawing process (because all rotations
;	are the same)
RotSet	proc
	xor ebx,ebx
	mov bl,[rotx]		;load point
	shl ebx,1		;bx = index in sintab and costab
	mov ax,[sintab + ebx] 	;find sine
	mov cx,[costab + ebx]	;find cos
	mov [sinx],ax		;
	xor ebx,ebx		;clear for y
	mov bl,[roty]		;load y
	mov [cosx],cx		;
	shl ebx,1		;find index
	mov cx,[sintab + ebx]	;store siny
	mov ax,[costab + ebx]
	xor ebx,ebx
	mov [siny],cx
	mov bl,[rotz]		; and sinaz & cosaz
	mov [cosy],ax
	shl ebx,1
	mov ax,[sintab + ebx]
	mov cx,[costab + ebx]
	mov [sinz],ax
	mov [cosz],cx

	ret
ENDP


;****************************************************************************
;	rotx,roty,rotz = angle to rotate (0 -> 255 in bl : need upper clear)
;	rx,ry,rz = points	(returns in these too) (words)
;	centx,centy,centz	(translate to center)
Rotate	proc

	push edi

	; Rotate point around the X-axis
		; ry = cos(angle x) * y - sin(angle x) * z
		; rz = sin(angle x) * y + cos(angle x) * z
	cmp [rotx],0	;see if need to rotate
	je SkipX
	mov cx,[ry]	;load real point
	mov ax,[cosx]	; ax = cos(angle x)
	imul cx		; dx:ax = cos(angle x) * y
	mov di,dx	; dx:ax -> edi
	shl edi,16	; is in dx and the low word in ax
	mov di,ax	; edi = by * cos(angle x)
	mov bx,[rz]	;load z
	mov ax,[sinx]	; ax = sin(angle x)
	imul bx		; ax = sin(angle x) * z
	shl edx,16
	mov dx,ax	; edx = bz * sinax
	sub edi,edx	; edi=edi-edx= cos(angle x)*y - sin(angle x)*z
	sar edi,14	; shift to the actual value (fixed point math)
	push di		; save rotated y
	mov ax,[sinx]	; ax = sin(angle x)
	imul cx		; dx:ax = sin(angle x) * y
	mov di,dx
	shl edi,16
	mov di,ax	; edi = sin(angle x) * y
	mov ax,[cosx]	; ax = cos(angle x)
	imul bx		; dx:ax = cos(angle x) * z
	shl edx,16
	mov dx,ax	; edx = cos(angle x) * z
	add edi,edx	;edi=edi+edx = sin(angle x)*y + cos(angle x)*z
	pop ax		;get ry
	sar edi,14	;shift right => di = actual value of rotated z
	mov [ry],ax	;restore y
	mov [rz],di	;
 SkipX:
	; Rotate point around the Y-axis
		; rx =  cos(angle y) * x + sin(angle y) * rz
		; rrz = -sin(angle y) * x + cos(angle y) * rz
	cmp [roty],0	;see if need to do
	je SkipY
	mov cx,[rx]	;
	mov ax,[cosy]	; ax = cos(angle y)
	imul cx		; dx:ax = cos(angle y) * x
	mov di,dx
	mov bx,[rz]
	shl edi,16
	mov di,ax	; edi = cos(angle y) * x
	mov ax,[siny]	; ax = sin(angle y)
	imul bx		; dx:ax = sin(angle y) * rz
	shl edx,16
	mov dx,ax	; edx = sin(angle y) * rz
	add edi,edx	;edi=edi+edx= cos(angle y)*x + sin(angle y)*rz
	sar edi,14	; apply fixed point math
	push di		; save rx (rotated x)
	mov ax,[siny]	; ax = sin(angle y)
	neg ax		; ax = -sin(angle y)
	imul cx		; dx:ax = -sin(angle y) * x
	mov di,dx
	shl edi,16
	mov di,ax	; edi = -sin(angle y) * x
	mov ax,[cosy]	; ax = cos(angle y)
	imul bx		; dx:ax = cos(angle y) * rz
	shl edx,16
	mov dx,ax	; edx = cos(angle y) * rz
	add edi,edx	;edi=edi-edx=-sin(angle y)*x + cos(angle y)*rz
	pop ax		;get rx
	sar edi,14	;fp math, di = rrz (rotated rz)
	mov [rx],ax	;restore
	mov [rz],di	;
 SkipY:
	; Rotate point around the Z-axis
		; rrx = cos(vz) * rx - sin(vz) * ry
		; rry = sin(vz) * rx + cos(vz) * ry
	cmp [rotz],0	;see if need to do
	je SkipZ	;
	mov bx,[rx]
	mov ax,[cosz]	; ax = cos(angle z)
	imul bx		; dx:ax = cos(angle z) * rx
	mov di,dx
	mov cx,[ry]
	shl edi,16
	mov di,ax	;edi = cos(angle z) * rx
	mov ax,[sinz]	; ax = sin(angle z)
	imul cx		; dx:ax = sin(angle z) * ry
	shl edx,16
	mov dx,ax	; edx = sin(angle z) * ry
	sub edi,edx	;edi=edi-edx=cos(angle z)*rx - sin(angle z)*ry
	sar edi,14	; shift right to actual value
	push di		;rrx
	mov ax,[sinz]	; ax = sin(angle z)
	imul bx		; dx:ax = sin(angle z) * rx
	mov di,dx
	shl edi,16
	mov di,ax	; edi = sin(angle z) * rx
	mov ax,[cosz]	; ax = cos(angle z)
	imul cx		; dx:ax = cos(angle z) * ry
	shl edx,16
	mov dx,ax	; edx = cos(angle z) * ry
	add edi,edx	;edi=edi+edx=sin(angle z)*rx + cos(angle z)*ry
	pop ax		;get rx
	sar edi,14	; shift right to actual value
	mov [rx],ax	;
	mov [ry],di	; cx = rry (rotated ry)
 SkipZ:
	pop edi
	ret
ENDP

include sqrt.inc